#ifndef PROTEIN_H
#define PROTEIN_H

#include "pointers.h"
#include <string>

namespace REFINE_NS{
  typedef enum {
	HH=1,
	LL=2,
	interH=3,
	interL=4,
	HL=5,
	ERROR=-1
  } CONTACTTYPE;

  class Protein : protected Pointers {
  public:

    int Taa;               // total number of amino acids
    int Tatom;             // total number of atoms of protein
    int *natom1;           // atom number of the first atom(N) in one amino acid

	double *sigma;			// for L-J potential
	double *epsilon;		// for L-J potential
	int *type;				// atom type of sTab4[][8] in protein_data.h
	int *type158;			// atom type 158 for RW

    char **AtomType;       // type of atom
    char **AA3;            // type of amino acid
    int *naa;              // residue number of this atom
    double **x;            // x y z coordinates
    double **x0;           // x y z coordinates backup 0 
    double **x00;          // x y z coordinates backup 00
    int *phi,*psi;         // phi psi list
    int *nclash;           // number of atomic clash for one residue
    //int RandomSeed;        // Random Seed
    int rw_flag;           // flag of rw reference state calculation
    int scb_flag;          // flag of side-chain orientation reference state calculation

    int bondout,angleout,ramaout; // # of bond, angle, rama outliers
    double bondE,angleE,ramaE;    // energy of bond, angle and rama

    double ***rwp,***scbp;        // rw and scb potential

    int ntm;
    int *tmb,*tme;     // transmembrane helix
    int *tmi;			// index of transmembrane helix
    double mbcx,mbcy,mbcz;//center point of the begin membrane plane
    double mecx,mecy,mecz;//center point of the end membrane plane
    double dmembrane;//the distance of the begin and end membrane
    double *membren_vector;    //the normal vector of begin membrane plane
    int *tmbnative,*tmenative;     // transmembrane helix for native
    int *tminative;			// index of transmembrane helix for native
    double *hydrophobicity; // hydrophobicity scale
    int *tmdv;			// direction vector of TM residues

	int *nnlist;		// neighbor list
	int **nlist;		// neighbor list

	double *sasa;		// solvent accessible surface area (sasa)
	int *nsalist;		// sasa neighbor list
	int **salist;		// sasa neighbor list

	double D100;	// 100 degree/180 degree * PI

	int ncontact;	// total # of contact restraint
	int **icontact;	// contact index of residue
	double *confidence_contact; //  confidence of the contact pair
	CONTACTTYPE  *icontacttype;	// type of the contact index of residue

	int ncontact_loop;	// total # of contact restraint
	int **icontact_loop;	// contact index of residue
	double *confidence_contact_loop; //  confidence of the contact pair
	CONTACTTYPE  *icontacttype_loop;	// type of the contact index of residue

	int ncontactnative;	// total # of contact restraint of the native
	int **icontactnative;	// contact index of residue of the native
	double *rcontact; // contact distance cut-off
	int *nresiduecontact; //number of contacts on the specified residue

	int **ncontacthelixmap;//contact map of helix,
	double *confidence_helix;//confidence of helix contact ,length=ntm
	double confidence_total;//the sum of total confidence


	//helix connection data
	double *helixconnectionAvg;
	double *helixconnectionDev;

	//helix angle data
	double **helixangleAvg;
	double **helixangleDev;

	//disulfide bond
	int ndisulfide;	// total # of disulfide restraint
	int **lst_disulfide;	//   index of disulfide residue

	//loop frag
	//1st dimension:residue index of the target loop segment in protein;
	//2nd dimension:residue index of choose fragment in fragment library;
	//target loop segments have different size of choose fragment
	int **loopfrag;

	//number of  fragments for ith segment
	int *loopfrag_number;

	//frag size for each fragments
	int FRAGSIZE;
    int MAXFRAG;
    // functions

	Protein(class REFINE *);
	~Protein();

    void init();
    //void rebuild_atom_type();      // rebuild the atom type, sigma, epsilon
    int read_pdb(char const  *);         // read in pdb file
	double calculate_rw();          // calculate RW potential
	double calculate_scb();         // calculate side-chain orientation potential
	void calculate_rw_reference();  // calculate random walk reference state
	void calculate_scb_reference(); // calculate side-chain orientation reference state
	int  block_type(char *);       // get the residue type
	int side_chain_block_type(char *); // side chain block types
	int scb_bin_num(double, double);   // bin number of side chain orientation potential
	void rebuild_atom_type();      // rebuild the atom type, sigma, epsilon
	double get_epsilon(char *, char *); // get the epsilon value for L-J potential
	double get_sigma(char *, char *);   // get the sigma value for L-J potential
	int  atom_type(char *);        // get the atom type
	int atom_type_158(char *, char *); // 158 atom types
  };
}

#endif
